from __future__ import division

import urllib
import json

from collections import deque

from pydie.enums import TYPE_UINT8
from pydie.enums import TYPE_UINT16
from pydie.enums import TYPE_HEX16
from pydie.enums import MAX_UINT8
from pydie.enums import MAX_UINT16

ANU_API_URL = 'https://qrng.anu.edu.au/API/jsonI.php'
MAX_BLOCK_SIZE = '1024'  # applies only to hex16
MAX_SET_LENGTH = '1024'

def fetch(uinttype, length=1, size=6):

    params = urllib.urlencode(
        {
            'type': uinttype,
            'size': min(int(size), MAX_BLOCK_SIZE),
            'length': min(int(length), MAX_SET_LENGTH)
        }
    )

    url = '?'.join((ANU_API_URL, params))

    response = json.loads(urllib.urlopen(url).read())

    if response['success'] is True:
        return response

    else:
        raise Exception("\n\nError encountered when requesting data: \n- url: {0}\n- response: {1}\n".format(
            url,
            response
        ))

def fetch_uint8(length=1):
    fetch(TYPE_UINT8, length=length)


def fetch_uint16(length=1):
    fetch(TYPE_UINT16, length=length)


def fetch_hex16(size=1, length=6):
    fetch(TYPE_HEX16, length=length, size=size)


def rand(min=0, max=1, length=1, uinttype=TYPE_UINT16, size=6):
    response = fetch(uinttype, length=length, size=size)
    numlist = deque(response['data'])

    # TODO: Revisit, small hack
    # - For whatever reason I need to subtract 1 from max to get numbers
    #   between min and max (w/o -1 it produced number over max)
    # - It "kinda" makes sense, but need to make sure this is
    #   working correctly
    modulus = get_uintmax(uinttype) / float(max-1)

    # flatten raw anu numbers to (min/max) range
    result = [x / modulus + min for x in numlist]

    return result

def randint(min=0, max=1, length=1, uinttype=TYPE_UINT16, size=6):
    # int(round(pick)) flattens floats that come back from anu.rand
    # python 2.7 round() returns a float :sadface:
    return [int(round(x)) for x in rand(**locals())]


def get_uintmax(uinttype):
    if uinttype == TYPE_UINT16:
        return MAX_UINT16

    elif uinttype == TYPE_UINT8:
        return MAX_UINT8

    else:
        raise Exception('Unsupported uinttype passed {0}'.format(uinttype))



import re

def parse_roll(rollstr):
    """Parse a roll
    """

    rDie = r'(?P<multiplier>\d+)d(?P<die>\d+)'
    rModifieres = r'\+\d|-\d'
    rDie = re.compile(rDie)
    rModifieres = re.compile(rModifieres)

    roll_and_modifiers = rollstr.split('m')
    roll = roll_and_modifiers[0]

    try:
        modifiers = roll_and_modifiers[1]

        modifiers = rModifieres.findall(modifiers)
    except:
        modifiers = []

    # TODO: Add error handling here
    roll_dict = rDie.match(roll).groupdict()
    roll_dict['modifiers'] = modifiers
    roll_dict['argv'] = rollstr

    return roll_dict



#!/usr/bin/env python

import os
import sys
from setuptools import setup

from pydie import get_version

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

os.environ['PYTHONDONTWRITEBYTECODE'] = '1'

# Setup arg values
url              = 'http://github.com/aubricus/pydie'
name             = 'pydie'
version          = get_version()
description      = 'Generate a random roll from a n-number of an n-sided die'
long_description = open('README.rst').read()
author           = 'Aubrey Taylor'
author_email     = 'aubricus@gmail.com'
zip_safe         = False

package_dir = {'pydie': 'pydie'}
packages = [
    'pydie'
]
package_data = {
    'pydie': ['LICENSE.md', 'README.md']
}

requires = [
    'setuptools',
    'docopt==0.6.1'
]

entry_points = {
    'console_scripts': ['pydie = pydie.cli:main']
}

classifiers = (
    'Development Status :: 2 - Pre-Alpha',
    'Intended Audience :: Developers',
    'Natural Language :: English',
    'License :: OSI Approved :: Apache Software License',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2.6',
    'Programming Language :: Python :: 2.7',
)

setup(
    url=url,
    name=name,
    version=version,
    description=description,
    author=author,
    author_email=author_email,
    packages=packages,
    package_data=package_data,
    install_requires=requires,
    entry_points=entry_points,
    package_dir=package_dir,
    long_description=long_description,
    classifiers=classifiers,
)

del os.environ['PYTHONDONTWRITEBYTECODE']



def parse_rolls_argv(rolls_argv):
    return [parse_roll_argv(argv) for argv in rolls_argv]


def parse_roll_argv(roll_argv):
    from pydie import parser
    return parser.parse_roll(roll_argv)


def do_roll(roll_dict, roll_method):
    return roll_method(roll_dict)


def do_rolls_from_argv(rolls_argv, roll_method):
    return [do_roll(roll, roll_method) for roll in parse_rolls_argv(rolls_argv)]


def roll(*args):
    from pydie import rollmethods
    return do_rolls_from_argv(args, rollmethods.anu_pick_n_shuffle)
